"""Automatically generated by hassfest.

To update, run python3 -m script.hassfest
"""

# fmt: off

FLOWS = [
    "abode",
    "accuweather",
    "acmeda",
    "adguard",
    "agent_dvr",
    "airly",
    "airvisual",
    "ais_dom_device",
    "ais_drives_service",
    "ais_google_home",
    "ais_host",
    "ais_spotify_service",
    "ais_supla",
    "ais_tauron",
    "ais_wear_os",
    "ais_wifi_service",
    "alarmdecoder",
    "almond",
    "ambiclimate",
    "ambient_station",
    "arcam_fmj",
    "atag",
    "august",
    "avri",
    "awair",
    "axis",
    "azure_devops",
    "blebox",
    "blink",
    "bond",
    "braviatv",
    "broadlink",
    "brother",
    "bsblan",
    "canary",
    "cast",
    "cert_expiry",
    "control4",
    "coolmaster",
    "coronavirus",
    "daikin",
    "deconz",
    "denonavr",
    "devolo_home_control",
    "dexcom",
    "dialogflow",
    "directv",
    "doorbird",
    "dunehd",
    "dynalite",
    "eafm",
    "ecobee",
    "elgato",
    "elkm1",
    "emulated_roku",
    "enocean",
    "esphome",
    "extalife",
    "fibaro",
    "flick_electric",
    "flo",
    "flume",
    "flunearyou",
    "forked_daapd",
    "freebox",
    "fritzbox",
    "garmin_connect",
    "gdacs",
    "geofency",
    "geonetnz_quakes",
    "geonetnz_volcano",
    "gios",
    "glances",
    "goalzero",
    "gogogate2",
    "gpslogger",
    "griddy",
    "guardian",
    "hangouts",
    "harmony",
    "heos",
    "hisense_aehw4a1",
    "hlk_sw16",
    "home_connect",
    "homekit",
    "homekit_controller",
    "homematicip_cloud",
    "huawei_lte",
    "hue",
    "hunterdouglas_powerview",
    "hvv_departures",
    "iaqualink",
    "icloud",
    "ifttt",
    "insteon",
    "ios",
    "ipma",
    "ipp",
    "iqvia",
    "islamic_prayer_times",
    "isy994",
    "izone",
    "juicenet",
    "kodi",
    "konnected",
    "life360",
    "lifx",
    "local_ip",
    "locative",
    "logi_circle",
    "luftdaten",
    "mailgun",
    "melcloud",
    "met",
    "meteo_france",
    "metoffice",
    "mikrotik",
    "mill",
    "minecraft_server",
    "mobile_app",
    "monoprice",
    "mqtt",
    "myq",
    "neato",
    "nest",
    "netatmo",
    "nexia",
    "nightscout",
    "notion",
    "nuheat",
    "nut",
    "nws",
    "nzbget",
    "omnilogic",
    "onvif",
    "opentherm_gw",
    "openuv",
    "openweathermap",
    "ovo_energy",
    "owntracks",
    "ozw",
    "panasonic_viera",
    "pi_hole",
    "plaato",
    "plex",
    "plugwise",
    "plum_lightpad",
    "point",
    "poolsense",
    "powerwall",
    "progettihwsw",
    "ps4",
    "pvpc_hourly_pricing",
    "rachio",
    "rainmachine",
    "ring",
    "risco",
    "roku",
    "roomba",
    "roon",
    "rpi_power",
    "samsungtv",
    "sense",
    "sentry",
    "sharkiq",
    "shelly",
    "shopping_list",
    "simplisafe",
    "smappee",
    "smart_meter_texas",
    "smarthab",
    "smartthings",
    "smhi",
    "sms",
    "solaredge",
    "solarlog",
    "soma",
    "somfy",
    "sonarr",
    "songpal",
    "sonos",
    "speedtestdotnet",
    "spider",
    "spotify",
    "squeezebox",
    "starline",
    "syncthru",
    "synology_dsm",
    "tado",
    "tellduslive",
    "tesla",
    "tibber",
    "tile",
    "toon",
    "totalconnect",
    "tplink",
    "traccar",
    "tradfri",
    "transmission",
    "tuya",
    "twentemilieu",
    "twilio",
    "unifi",
    "upb",
    "upnp",
    "velbus",
    "vera",
    "vesync",
    "vilfo",
    "vizio",
    "volumio",
    "wemo",
    "wiffi",
    "wilight",
    "withings",
    "wled",
    "wolflink",
    "xiaomi_aqara",
    "xiaomi_miio",
    "yeelight",
    "zerproc",
    "zha",
    "zwave"
]
