"""Constants for the Azure DevOps integration."""
DOMAIN = "azure_devops"

DATA_AZURE_DEVOPS_CLIENT = "azure_devops_client"
DATA_ORG = "organization"
DATA_PROJECT = "project"
DATA_PAT = "personal_access_token"

CONF_ORG = "organization"
CONF_PROJECT = "project"
CONF_PAT = "personal_access_token"
